@@ -13,6 +13,7 @@ from dataclasses import dataclass
 from datetime import datetime
 from enum import Enum
 from typing import Any, Collection, Optional
+from web3 import Web3
 
 from dotenv import load_dotenv
 
@@ -30,6 +31,61 @@ DictInnerResponse = dict[str, dict[str, Any]]
 
 DuneRecord = dict[str, str]
 
+# pylint: disable=too-few-public-methods
+class Address:
+    """
+    Class representing Ethereum Address as a hexadecimal string of length 42.
+    The string must begin with '0x' and the other 40 characters
+    are digits 0-9 or letters a-f. Upon creation (from string) addresses
+    are validated and stored in their check-summed format.
+    """
+
+    def __init__(self, address: str):
+        # Dune uses \x instead of 0x (i.e. bytea instead of hex string)
+        # This is just a courtesy to query writers,
+        # so they don't have to convert all addresses to hex strings manually
+        address = address.replace("\\x", "0x")
+        if Address._is_valid(address):
+            self.address: str = Web3.toChecksumAddress(address)
+        else:
+            raise ValueError(f"Invalid Ethereum Address {address}")
+
+    def __str__(self) -> str:
+        return str(self.address)
+
+    def __eq__(self, other: object) -> bool:
+        if isinstance(other, Address):
+            return self.address == other.address
+        return False
+
+    def __lt__(self, other: object) -> bool:
+        if isinstance(other, Address):
+            return str(self).lower() < str(other).lower()
+        return False
+
+    def __hash__(self) -> int:
+        return self.address.__hash__()
+
+    @classmethod
+    def zero(cls) -> Address:
+        """Returns Null Ethereum Address"""
+        return cls("0x0000000000000000000000000000000000000000")
+
+    @classmethod
+    def from_int(cls, num: int) -> Address:
+        """
+        Construct an address from int.
+        Used for testing, so that 123 -> "0x0000000000000000000000000000000000000123"
+        """
+        return cls(f"0x{str(num).rjust(40, '0')}")
+
+    @staticmethod
+    def _is_valid(address: str) -> bool:
+        match_result = re.match(
+            pattern=r"^(0x)?[0-9a-f]{40}$", string=address, flags=re.IGNORECASE
+        )
+        return match_result is not None
+
 
 # pylint: disable=too-few-public-methods
 # TODO - use namedtuple for MetaData and QueryResults
@@ -141,6 +197,7 @@ class ParameterType(Enum):
     TEXT = "text"
     NUMBER = "number"
     DATE = "datetime"
+    ENUM = "enum"
 
     @classmethod
     def from_string(cls, type_str: str) -> ParameterType:
@@ -152,10 +209,11 @@ class ParameterType(Enum):
             r"text": cls.TEXT,
             r"number": cls.NUMBER,
             r"date": cls.DATE,
+            r"enum": cls.ENUM,
         }
-        for pattern, network in patterns.items():
+        for pattern, param in patterns.items():
             if re.match(pattern, type_str, re.IGNORECASE):
-                return network
+                return param
         raise ValueError(f"could not parse Network from '{type_str}'")
 
 
@@ -167,10 +225,14 @@ class QueryParameter:
         name: str,
         parameter_type: ParameterType,
         value: Any,
+        options: Optional[list[str]] = None,
     ):
+        if not options:
+            options = []
         self.key: str = name
         self.type: ParameterType = parameter_type
         self.value = value
+        self.options = options
 
     def __eq__(self, other: object) -> bool:
         if not isinstance(other, QueryParameter):
@@ -203,21 +265,28 @@ class QueryParameter:
             value = postgres_date(value)
         return cls(name, ParameterType.DATE, value)
 
+    @classmethod
+    def enum_type(cls, name: str, value: str, options: list[str]) -> QueryParameter:
+        """Constructs a Query parameter of type number"""
+        return cls(name, ParameterType.ENUM, value, options)
+
     def _value_str(self) -> str:
-        if self.type in (ParameterType.TEXT, ParameterType.NUMBER):
+        if self.type in (ParameterType.TEXT, ParameterType.NUMBER, ParameterType.ENUM):
             return str(self.value)
         if self.type == ParameterType.DATE:
             # This is the postgres string format of timestamptz
             return str(self.value.strftime("%Y-%m-%d %H:%M:%S"))
         raise TypeError(f"Type {self.type} not recognized!")
 
-    def to_dict(self) -> dict[str, str]:
+    def to_dict(self) -> dict[str, str | list[str]]:
         """Converts QueryParameter into string json format accepted by Dune API"""
-        results = {
+        results: dict[str, str | list[str]] = {
             "key": self.key,
             "type": self.type.value,
             "value": self._value_str(),
         }
+        if self.type == ParameterType.ENUM:
+            results["enumOptions"] = self.options
         return results
 
     @classmethod
@@ -237,6 +306,8 @@ class QueryParameter:
             if isinstance(value, str):
                 value = float(value) if "." in value else int(value)
             return cls.number_type(name, value)
+        if p_type == ParameterType.ENUM:
+            return cls.enum_type(name, value, obj["enumOptions"])
         raise ValueError(f"Could not parse Query parameter from {obj}")
 
     def __str__(self) -> str:
@@ -352,7 +423,7 @@ class DuneQuery:
             query_id=tile.query_id,
         )
 
-    def _request_parameters(self) -> list[dict[str, str]]:
+    def _request_parameters(self) -> list[dict[str, str | list[str]]]:
         return [p.to_dict() for p in self.parameters]
 
     def upsert_query_post(self) -> Post:
